@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2013 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #define _XOPEN_SOURCE
 #define _XOPEN_SOURCE_EXTENDED 1
 #include "weather-parsers.h"
+#include "weather-translate.h"
 #include "weather-debug.h"
 
 #include <time.h>
@@ -94,9 +95,34 @@ get_timeslice(xml_weather *wd,
 }
 
 
+xml_astro *
+get_astro(const GArray *astrodata,
+          const time_t day_t,
+          guint *index)
+{
+    xml_astro *astro;
+    gint i;
+
+    g_assert(astrodata != NULL);
+    if (G_UNLIKELY(astrodata == NULL))
+        return NULL;
+
+    for (i = 0; i < astrodata->len; i++) {
+        astro = g_array_index(astrodata, xml_astro *, i);
+        if (astro && astro->day == day_t) {
+            if (index != NULL)
+                *index = i;
+            return astro;
+        }
+    }
+    return NULL;
+}
+
+
 time_t
 parse_timestring(const gchar *ts,
-                 gchar *format) {
+                 gchar *format,
+                 gboolean local) {
     time_t t;
     struct tm tm;
 
@@ -116,7 +142,11 @@ parse_timestring(const gchar *ts,
     if (G_UNLIKELY(strptime(ts, format, &tm) == NULL))
         return t;
 
-    t = my_timegm(&tm);
+    if (local)
+        t = mktime(&tm);
+    else
+        t = my_timegm(&tm);
+
     return t;
 }
 
@@ -196,8 +226,8 @@ parse_location(xmlNode *cur_node,
         }
         if (NODE_IS_TYPE(child_node, "symbol")) {
             g_free(loc->symbol);
-            loc->symbol = PROP(child_node, "id");
             loc->symbol_id = strtol(PROP(child_node, "number"), NULL, 10);
+            loc->symbol = g_strdup(get_symbol_for_id(loc->symbol_id));
         }
     }
 
@@ -268,11 +298,11 @@ parse_time(xmlNode *cur_node,
     xmlFree(datatype);
 
     from = PROP(cur_node, "from");
-    start_t = parse_timestring(from, NULL);
+    start_t = parse_timestring(from, NULL, FALSE);
     xmlFree(from);
 
     to = PROP(cur_node, "to");
-    end_t = parse_timestring(to, NULL);
+    end_t = parse_timestring(to, NULL, FALSE);
     xmlFree(to);
 
     if (G_UNLIKELY(!start_t || !end_t))
@@ -363,11 +393,11 @@ parse_astro_location(xmlNode *cur_node,
             xmlFree(never_sets);
 
             sunrise = PROP(child_node, "rise");
-            astro->sunrise = parse_timestring(sunrise, NULL);
+            astro->sunrise = parse_timestring(sunrise, NULL, FALSE);
             xmlFree(sunrise);
 
             sunset = PROP(child_node, "set");
-            astro->sunset = parse_timestring(sunset, NULL);
+            astro->sunset = parse_timestring(sunset, NULL, FALSE);
             xmlFree(sunset);
         }
 
@@ -391,11 +421,11 @@ parse_astro_location(xmlNode *cur_node,
             xmlFree(never_sets);
 
             moonrise = PROP(child_node, "rise");
-            astro->moonrise = parse_timestring(moonrise, NULL);
+            astro->moonrise = parse_timestring(moonrise, NULL, FALSE);
             xmlFree(moonrise);
 
             moonset = PROP(child_node, "set");
-            astro->moonset = parse_timestring(moonset, NULL);
+            astro->moonset = parse_timestring(moonset, NULL, FALSE);
             xmlFree(moonset);
 
             astro->moon_phase = PROP(child_node, "phase");
@@ -404,37 +434,57 @@ parse_astro_location(xmlNode *cur_node,
 }
 
 
-/*
- * Look at http://api.yr.no/weatherapi/sunrise/1.0/schema for information
- * of elements and attributes to expect.
- */
-xml_astro *
-parse_astro(xmlNode *cur_node)
+static xml_astro *
+parse_astro_time(xmlNode *cur_node)
 {
-    xmlNode *child_node, *time_node = NULL;
+    xmlNode *child_node;
     xml_astro *astro;
-
-    g_assert(cur_node != NULL);
-    if (G_UNLIKELY(cur_node == NULL || !NODE_IS_TYPE(cur_node, "astrodata")))
-        return NULL;
+    gchar *date;
 
     astro = g_slice_new0(xml_astro);
     if (G_UNLIKELY(astro == NULL))
         return NULL;
 
+    date = PROP(cur_node, "date");
+    astro->day = parse_timestring(date, "%Y-%m-%d", TRUE);
+    xmlFree(date);
+
+    for (child_node = cur_node->children; child_node;
+         child_node = child_node->next)
+        if (NODE_IS_TYPE(child_node, "location"))
+            parse_astro_location(child_node, astro);
+    return astro;
+}
+
+
+/*
+ * Look at http://api.yr.no/weatherapi/sunrise/1.0/schema for information
+ * of elements and attributes to expect.
+ */
+gboolean
+parse_astrodata(xmlNode *cur_node,
+                GArray *astrodata)
+{
+    xmlNode *child_node;
+    xml_astro *astro;
+
+    g_assert(astrodata != NULL);
+    if (G_UNLIKELY(astrodata == NULL))
+        return FALSE;
+
+    g_assert(cur_node != NULL);
+    if (G_UNLIKELY(cur_node == NULL || !NODE_IS_TYPE(cur_node, "astrodata")))
+        return FALSE;
+
     for (child_node = cur_node->children; child_node;
          child_node = child_node->next)
         if (NODE_IS_TYPE(child_node, "time")) {
-            time_node = child_node;
-            break;
+            if ((astro = parse_astro_time(child_node))) {
+                merge_astro(astrodata, astro);
+                xml_astro_free(astro);
+            }
         }
-
-    if (G_LIKELY(time_node))
-        for (child_node = time_node->children; child_node;
-             child_node = child_node->next)
-            if (NODE_IS_TYPE(child_node, "location"))
-                parse_astro_location(child_node, astro);
-    return astro;
+    return TRUE;
 }
 
 
@@ -613,6 +663,36 @@ xml_location_free(xml_location *loc)
 }
 
 
+/*
+ * Deep copy xml_astro struct.
+ */
+xml_astro *
+xml_astro_copy(const xml_astro *src)
+{
+    xml_astro *dst;
+
+    if (G_UNLIKELY(src == NULL))
+        return NULL;
+
+    dst = g_slice_new0(xml_astro);
+    g_assert(dst != NULL);
+    if (G_UNLIKELY(dst == NULL))
+        return NULL;
+
+    dst->day = src->day;
+    dst->sunrise = src->sunrise;
+    dst->sunset = src->sunset;
+    dst->sun_never_rises = src->sun_never_rises;
+    dst->sun_never_sets = src->sun_never_sets;
+    dst->moonrise = src->moonrise;
+    dst->moonset = src->moonset;
+    dst->moon_never_rises = src->moon_never_rises;
+    dst->moon_never_sets = src->moon_never_sets;
+    dst->moon_phase = g_strdup(src->moon_phase);
+    return dst;
+}
+
+
 /*
  * Deep copy xml_time struct.
  */
@@ -623,16 +703,20 @@ xml_time_copy(const xml_time *src)
     xml_location *loc;
     gint i;
 
-    if (src == NULL)
+    if (G_UNLIKELY(src == NULL))
         return NULL;
 
     dst = g_slice_new0(xml_time);
-    if (dst == NULL)
+    g_assert(dst != NULL);
+    if (G_UNLIKELY(dst == NULL))
         return NULL;
 
     loc = g_slice_new0(xml_location);
-    if (loc == NULL)
-        return dst;
+    g_assert(loc != NULL);
+    if (loc == NULL) {
+        g_slice_free(xml_time, dst);
+        return NULL;
+    }
 
     dst->start = src->start;
     dst->end = src->end;
@@ -744,6 +828,23 @@ xml_astro_free(xml_astro *astro)
 }
 
 
+void
+astrodata_free(GArray *astrodata)
+{
+    xml_astro *astro;
+    gint i;
+
+    if (G_UNLIKELY(astrodata == NULL))
+        return;
+    for (i = 0; i < astrodata->len; i++) {
+        astro = g_array_index(astrodata, xml_astro *, i);
+        if (astro)
+            xml_astro_free(astro);
+    }
+    g_array_free(astrodata, FALSE);
+}
+
+
 void
 xml_geolocation_free(xml_geolocation *geo)
 {
